----------------------------------------------------------------------------------------------------
-- (c)2007 Altera Corporation. All rights reserved.
--
-- Altera products are protected under numerous U.S. and foreign patents,
-- maskwork rights, copyrights and other intellectual property laws.
--
-- This reference design file, and your use thereof, is subject to and governed
-- by the terms and conditions of the applicable Altera Reference Design License
-- Agreement (either as signed by you or found at www.altera.com).  By using
-- this reference design file, you indicate your acceptance of such terms and
-- conditions between you and Altera Corporation.  In the event that you do not
-- agree with such terms and conditions, you may not use the reference design
-- file and please promptly destroy any copies you have made.
--
-- This reference design file is being provided on an "as-is�" basis and as an
-- accommodation and therefore all warranties, representations or guarantees of
-- any kind (whether express, implied or statutory) including, without
-- limitation, warranties of merchantability, non-infringement, or fitness for
-- a particular purpose, are specifically disclaimed.  By making this reference
-- design file available, Altera expressly does not recommend, suggest or
-- require that this reference design file be used in combination with any
-- other product not provided by Altera.
----------------------------------------------------------------------------------------------------
-- File          : superliteii_txrx_module.vhd
-- Author		 : Peter Schepers
----------------------------------------------------------------------------------------------------
-- Contains :
-- superliteii_txrx_module V3 : 	
--										- Instantiates 64b66b encoder/decoder + scrambling
--										- Deskew the lanes using RX FIFO inside the transceiver
--
--
-- Native PHY and reset controller. 
----------------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use work.package_delaytype.all; -- package that define "Delay_Array" type
 
entity superliteii_txrx_module is
GENERIC
	(
		READ_LENGTH			: std_logic_vector(15 downto 0) := X"00FF";
		IDLE_LENGTH			: std_logic_vector(3 downto 0)  := X"7";		
		NUMBER_OF_LANES				: integer := 10;	  	-- NUMBER_OF_LANES
		LANEWIDTH					: integer := 64	-- LANEWIDTH for transceiver
	);
PORT(
	Reset						: in std_logic;	
	RefClock					: in std_logic; 						-- RefClock Transceiver 
	Clk100Mhz				: in std_logic;						-- Required as reconfig clock and calibration clock
	
	-- Tx Path
	DataIn						: in std_logic_vector(((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);
	DataIn_Valid				: in std_logic; 					-- Only write when DataIn_valid is high	
	XOFF							: in std_logic;		-- XOFF input to sent out XOFF to the remote device for the remote to stop sending traffic (100 Mhz Clock domain)
	DataClock					: in std_logic;
	DataIn_ready				: out std_logic;			-- Asserted when FIFO can accept new data;
	
	Reset_Tx						: buffer std_logic;
	TxCoreClock					: buffer std_logic;
	
	
	-- High speed serial data outputs	
	XCVR_TX					: out std_logic_vector((NUMBER_OF_LANES-1) DOWNTO 0); 	  	
	
	-- Status signals	
	pll_locked					: buffer std_logic_vector(0 downto 0);
	wrfull_Tx_Ratematcher		: out std_logic;	
	tx_ready						: buffer std_logic;

   -- Rx Path
	
	ForceAlign					: in std_logic;	-- Force the Rx_Path to re-align
	Reset_Rx						: buffer std_logic;
	
	-- High speed serial input data lanes
	XCVR_RX						: in std_logic_vector((NUMBER_OF_LANES-1) DOWNTO 0); 
   Serial_Loop					: in std_logic_vector((NUMBER_OF_LANES-1) DOWNTO 0); 

	
	-- Parallel Data Output (with Valid signal) and clock
	DataOut						: out std_logic_vector(((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);
	DataOut_Valid				: out std_logic; 
	DataOut_Clock				: buffer std_logic;  
			
	-- Transceiver reconfiguration interface	
	reconfig_reset          : in  std_logic; 
	reconfig_write          : in  std_logic; 
	reconfig_read           : in  std_logic; 
	reconfig_address        : in  std_logic_vector(15 downto 0)  := (others => '0'); 
	reconfig_writedata      : in  std_logic_vector(31 downto 0)  := (others => '0'); 
	reconfig_readdata       : out std_logic_vector(31 downto 0);   
	reconfig_waitrequest    : out  std_logic;
	-- Tx PLL reconfiguration interface	
	reconfig_pll_reset          : in  std_logic; 
	reconfig_pll_write          : in  std_logic; 
	reconfig_pll_read           : in  std_logic; 
	reconfig_pll_address        : in  std_logic_vector(15 downto 0)  := (others => '0'); 
	reconfig_pll_writedata      : in  std_logic_vector(31 downto 0)  := (others => '0'); 
	reconfig_pll_readdata       : out std_logic_vector(31 downto 0);   
	reconfig_pll_waitrequest    : out  std_logic;

  
	-- Status signals	
	rx_freqlocked				: buffer std_logic_vector((NUMBER_OF_LANES-1) DOWNTO 0); 	
   rx_ready						: buffer std_logic;	
	Reset_Rx_Path				: buffer std_logic;
	WordAligned					: buffer std_logic;
	LaneAligned					: buffer std_logic;	
	Linkup						: buffer std_logic;
	XOFF_Received				: buffer std_logic;	
	Rx_Error						: buffer std_logic;
	decoder_error				: out std_logic;	
	Error_Deskew				: out std_logic;
	Delay							: buffer Delay_Array		
	);
END superliteii_txrx_module;


architecture rtl of superliteii_txrx_module is



component Reset_Synchro
port
	(
	Clk			: in std_logic;
	Reset_in	: in std_logic;
	Reset_out	: out std_logic
	);
end component;

component Tx_Ratematcher_Control
GENERIC
	(
		READ_LENGTH			: std_logic_vector(15 downto 0) := X"00FF";
		IDLE_LENGTH			: std_logic_vector(3 downto 0)  := X"7";		
		NUMBER_OF_LANES	: integer := 10;	  	-- NUMBER_OF_LANES
		LANEWIDTH	: integer := 64;		-- LANEWIDTH for transceiver
		RMFIFO_READ_ON_TH  				: std_logic_vector(4 downto 0) := "10100"; 	-- Ratematching fifo read on threshold: read starts when rlev is greater than this threshold
		RMFIFO_READ_OFF_TH 				: std_logic_vector(4 downto 0) := "00011" 	-- Ratematching fifo read off threshold: read stops when rlev is less than this threshold
		
	);
PORT(
	data_in						: in std_logic_vector(((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);
	data_in_valid			: in std_logic;
	data_in_ready			: out std_logic;
	wrclk						: in std_logic;							
	rdclk						: in std_logic;							
	Reset						: in std_logic;
   LaneAligned					: in std_logic; 			-- LaneAligned status from the receiver (clocked by recovered clock)
   XOFF							: in std_logic; 			-- XOFF input to sent out XOFF to the remote device for the remote to stop sending traffic (100 Mhz Clock domain)	
	Stop_Traffic				: in std_logic;			-- Used to stop sending traffic locally (when XOFF is received from remote side) (clocked by recovered clock)	
	Tx_Data						: out std_logic_vector(((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);
	Tx_Ctrlenable				: out std_logic_vector(((NUMBER_OF_LANES * (LANEWIDTH/8))-1) downto 0);
	wrfull_Tx_Ratematcher		: out std_logic
	);
END component;	


component txrx_pcs_64b66b is
GENERIC
	(
		NUMBER_OF_LANES	: integer := 10;	  	-- NUMBER_OF_LANES
		LANEWIDTH			: integer := 64		-- LANEWIDTH for transceiver
	);
PORT(
		Reset						: IN STD_LOGIC ;
		Serial_Loop				: IN STD_LOGIC_VECTOR ((NUMBER_OF_LANES - 1) downto 0);
		reconfig_mgmt_clk		: in std_logic; 
		pll_inclk				: IN STD_LOGIC ;
		tx_coreclk				: IN STD_LOGIC_VECTOR ((NUMBER_OF_LANES - 1) downto 0);
		tx_ctrlenable			: IN STD_LOGIC_VECTOR (((NUMBER_OF_LANES * (LANEWIDTH/8))-1) downto 0);
		tx_datain				: IN STD_LOGIC_VECTOR (((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);
		pll_locked				: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_clkout				: OUT STD_LOGIC_VECTOR ((NUMBER_OF_LANES - 1) downto 0);
		tx_dataout				: OUT STD_LOGIC_VECTOR ((NUMBER_OF_LANES - 1) downto 0);		
		rx_datain				: IN STD_LOGIC_VECTOR((NUMBER_OF_LANES-1) DOWNTO 0); 
	   rx_fifo_rd_en			: in 	STD_LOGIC_VECTOR((NUMBER_OF_LANES-1) DOWNTO 0); 
		rx_enh_fifo_pempty	: out STD_LOGIC_VECTOR((NUMBER_OF_LANES-1) DOWNTO 0);		
		Force_Rx_Reset			: in std_logic;
		rx_clkout				: OUT STD_LOGIC_VECTOR ((NUMBER_OF_LANES - 1) downto 0);
		rx_ctrldetect			: OUT STD_LOGIC_VECTOR (((NUMBER_OF_LANES * (LANEWIDTH/8))-1) downto 0);
		rx_dataout				: OUT STD_LOGIC_VECTOR (((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);
		rx_disperr				: OUT STD_LOGIC_VECTOR (((NUMBER_OF_LANES * (LANEWIDTH/8))-1) downto 0);
		rx_errdetect			: OUT STD_LOGIC_VECTOR (((NUMBER_OF_LANES * (LANEWIDTH/8))-1) downto 0);
		rx_freqlocked			: OUT STD_LOGIC_VECTOR ((NUMBER_OF_LANES-1) DOWNTO 0);
		rx_valid					: OUT STD_LOGIC_VECTOR ((NUMBER_OF_LANES-1) DOWNTO 0);
		Aligned					: OUT STD_LOGIC_VECTOR ((NUMBER_OF_LANES-1) DOWNTO 0);
		encoder_error			: OUT STD_LOGIC_VECTOR ((NUMBER_OF_LANES-1) DOWNTO 0);
		decoder_error			: OUT STD_LOGIC_VECTOR ((NUMBER_OF_LANES-1) DOWNTO 0);	
		tx_ready					: BUFFER STD_LOGIC;
		rx_ready					: BUFFER STD_LOGIC;
		-- Transceiver reconfiguration interface	
		reconfig_reset          : in  std_logic; 
		reconfig_write          : in  std_logic; 
		reconfig_read           : in  std_logic; 
		reconfig_address        : in  std_logic_vector(15 downto 0)  := (others => '0'); 
		reconfig_writedata      : in  std_logic_vector(31 downto 0)  := (others => '0'); 
		reconfig_readdata       : out std_logic_vector(31 downto 0);   
		reconfig_waitrequest    : out  std_logic;
		-- transmit PLL reconfiguration interface	
		reconfig_pll_reset          : in  std_logic; 
		reconfig_pll_write          : in  std_logic; 
		reconfig_pll_read           : in  std_logic; 
		reconfig_pll_address        : in  std_logic_vector(15 downto 0)  := (others => '0'); 
		reconfig_pll_writedata      : in  std_logic_vector(31 downto 0)  := (others => '0'); 
		reconfig_pll_readdata       : out std_logic_vector(31 downto 0);   
		reconfig_pll_waitrequest    : out  std_logic		
	);
end component;


component Rx_Path_Deskew 
GENERIC
	(
		NUMBER_OF_LANES	: integer := 5;	  	-- NUMBER_OF_LANES
		LANEWIDTH		: integer := 64		-- LANEWIDTH for transceiver
	);
PORT(
	Clock						: in std_logic;							
	Reset						: in std_logic;
	data_in						: in std_logic_vector(((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);
	ctrl_in						: in std_logic_vector((NUMBER_OF_LANES-1) DOWNTO 0);  		
	data_out						: out std_logic_vector(((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);
	rx_fifo_rd_en				: out STD_LOGIC_VECTOR((NUMBER_OF_LANES-1) DOWNTO 0); 
	rx_enh_fifo_pempty		: in STD_LOGIC_VECTOR((NUMBER_OF_LANES-1) DOWNTO 0);
	Valid_Data					: out std_logic;
	Error							: out std_logic;
	Force_Rx_Reset				: out std_logic;		-- This signal must be synchronized to the CLk100Mhz domain externally and used to trigger the Reset of Receiver Reset Instance
	Lane_Aligned				: buffer std_logic;
	Linkup						: buffer std_logic;
	XOFF_Received				: buffer std_logic;	
	Rx_Error						: buffer std_logic;	
	Delay							: buffer Delay_Array
	);
END component;	

type    Bit4_Type is array (0 to (NUMBER_OF_LANES - 1)) of std_logic_vector(3 downto 0);

signal tx_data 						: std_logic_vector(((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);
signal training_data					: std_logic_vector(((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);

signal tx_ctrlenable					: std_logic_vector(((NUMBER_OF_LANES * (LANEWIDTH/8))-1) downto 0);
signal tx_ctrlenable_training		: std_logic_vector(((NUMBER_OF_LANES * (LANEWIDTH/8))-1) downto 0);

signal tx_coreclk						: std_logic_vector((NUMBER_OF_LANES - 1) downto 0);
signal tx_clkout						: std_logic_vector((NUMBER_OF_LANES - 1) downto 0);

signal pll_powerdown					: std_logic_vector((NUMBER_OF_LANES - 1) downto 0);
signal tx_cal_busy					: std_logic_vector((NUMBER_OF_LANES - 1) downto 0);
signal tx_ready_i						: std_logic_vector((NUMBER_OF_LANES - 1) downto 0);
signal rx_ready_i						: std_logic_vector((NUMBER_OF_LANES - 1) downto 0);
signal rx_cal_busy					: std_logic_vector((NUMBER_OF_LANES - 1) downto 0);

signal Reset_in1						: std_logic;
signal Reset_in2						: std_logic;

signal count							: std_logic_vector(3 downto 0);

signal rx_cruclk			: std_logic_vector((NUMBER_OF_LANES - 1) downto 0);

signal rx_errdetect			: std_logic_vector(((NUMBER_OF_LANES * (LANEWIDTH/8))-1) downto 0);
signal rx_ctrldetect 		: std_logic_vector(((NUMBER_OF_LANES * (LANEWIDTH/8))-1) downto 0);
signal rx_disperr			: std_logic_vector(((NUMBER_OF_LANES * (LANEWIDTH/8))-1) downto 0);

signal rx_out 				: std_logic_vector(((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);
signal rx_clkout			: std_logic_vector((NUMBER_OF_LANES - 1) downto 0);
signal rx_coreclk			: std_logic_vector((NUMBER_OF_LANES - 1) downto 0);


signal DataOut_Aligned 		: std_logic_vector(((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);
signal Ctrl_Aligned			: std_logic_vector((NUMBER_OF_LANES - 1) downto 0);
signal Word_Aligned			: std_logic_vector((NUMBER_OF_LANES - 1) downto 0);


signal rx_freqlocked_Q		: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal rx_freqlocked_Q1		: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal rx_freqlocked_Q2		: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal rx_freqlocked_Q3		: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal Freqlock_Achieved	: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal rx_analogreset_i		: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal rx_digitalreset_i	: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal tx_analogreset_i		: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal tx_digitalreset_i	: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal Reset_I				: std_logic;


signal Zeroes 				: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal Ones 				: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal Reset_Lanes			: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);

signal RxCoreClock			: std_logic;

signal ForceAlign_min2			: std_logic;
signal ForceAlign_min1			: std_logic;
signal ForceAlign_sync			: std_logic;



signal SecondAlignment_Found  : std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal Ones_4times					: std_logic_vector (((NUMBER_OF_LANES * (LANEWIDTH/8))-1) downto 0);
signal Zeroes_4times					: std_logic_vector (((NUMBER_OF_LANES * (LANEWIDTH/8))-1) downto 0);

signal Reset_xcvr_rx				 : std_logic;
signal Force_Rx_Reset			 : std_logic;
signal force_xcvr_reset_rx		 : std_logic;
signal synchro						 : std_logic_vector(3 downto 0);
signal reset_rx_count			 : std_logic_vector(3 downto 0);	
signal rx_fifo_rd_en				 : std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);

signal rx_valid			: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal encoder_error		: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal decoder_error_i	: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);

signal rx_enh_fifo_pempty : std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);


begin

----------------------------------------------------------------------------------------
-- Generate Reset_Tx
----------------------------------------------------------------------------------------

Reset_in1 <= Reset OR not (tx_ready);		-- Both are generated on Clk100Mhz

Reset_Synchro_inst1 : Reset_Synchro
PORT MAP
	(
	Clk			=> DataClock,
	Reset_in		=> Reset_in1,
	Reset_out	=> Reset_Tx
	);

----------------------------------------------------------------------------------------
-- Tx Ratematcher Control (Superlite II TX)
----------------------------------------------------------------------------------------

TxCoreClock <= tx_clkout(0);



Tx_Ratematcher_Control_inst: Tx_Ratematcher_Control 
	GENERIC MAP
	(
		READ_LENGTH				=> READ_LENGTH,
		IDLE_LENGTH				=> IDLE_LENGTH,		
		NUMBER_OF_LANES		=> NUMBER_OF_LANES,	  	-- NUMBER_OF_LANES
		LANEWIDTH				=> LANEWIDTH,		-- LANEWIDTH for transceiver
		RMFIFO_READ_ON_TH  	=> "10100", 	-- Ratematching fifo read on threshold: read starts when rlev is greater than this threshold
		RMFIFO_READ_OFF_TH 	=> "00100" 	-- Ratematching fifo read off threshold: read stops when rlev is less than this threshold
		
	)
	PORT MAP( 
	data_in						=> DataIn, 
	data_in_valid				=> DataIn_valid,
	data_in_ready				=> DataIn_ready,	
	wrclk							=> DataClock,						
	rdclk							=> TxCoreClock,					
	Reset							=> Reset_Tx,
   LaneAligned					=> LaneAligned,
   XOFF							=> XOFF,
	Stop_Traffic				=> XOFF_Received,	
	Tx_Data						=> tx_data,
	Tx_Ctrlenable				=> tx_ctrlenable,
	wrfull_Tx_Ratematcher		=> wrfull_Tx_Ratematcher
	);
	
	

-----------------------------------------------------------------------------------------------------------
--	Generate Zeroes and Ones
-----------------------------------------------------------------------------------------------------------	

	
Generate_ZEROES_and_ONES:
FOR i IN 0 to NUMBER_OF_LANES-1 GENERATE
ONES(I) <= '1';
ZEROES(I) <= '0';
END GENERATE;

Generate_Ones_Zeroes_4times:
FOR i IN 0 to (NUMBER_OF_LANES*4)-1 GENERATE
	Ones_4times(I) <= '1';
	Zeroes_4times(I) <= '0';
End Generate;

	
		
-----------------------------------------------------------------------------------------------------------
--	Instantiate TxRx PCS 64b66b
-----------------------------------------------------------------------------------------------------------	


Generate_xcvr_tx_clocks:
FOR i IN 0 to NUMBER_OF_LANES-1 GENERATE
	tx_coreclk(I) 	<= tx_clkout(0);   
	--rx_coreclk(I) 	<= rx_clkout(0);   -- Use rx_clkout of lane 0.
   --pll_powerdown_c(I) <= pll_powerdown(0);	
END GENERATE;

DataOut_Clock <= rx_clkout(0);

xcvr_txrx_inst : txrx_pcs_64b66b 
	GENERIC MAP
		(
		NUMBER_OF_LANES		=> NUMBER_OF_LANES,	  	-- NUMBER_OF_LANES
		LANEWIDTH				=> LANEWIDTH		-- LANEWIDTH for transceiver
		)
	PORT MAP (
		Reset						=> Reset,

		Serial_Loop				=> Serial_Loop,
		reconfig_mgmt_clk		=> Clk100Mhz,	
		pll_inclk	 			=> RefClock,
	
		tx_coreclk	 			=> tx_coreclk,
		tx_ctrlenable	 		=> tx_ctrlenable,
		tx_datain	 			=> tx_data,
		pll_locked	 			=> pll_locked,
		tx_clkout	 			=> tx_clkout,
		tx_dataout	 			=> XCVR_TX,		
		rx_datain				=> XCVR_RX,
		rx_fifo_rd_en			=> rx_fifo_rd_en,
		rx_enh_fifo_pempty	=> rx_enh_fifo_pempty,	
		Force_Rx_Reset			=> Force_Rx_Reset,
		rx_clkout				=> rx_clkout,
		rx_ctrldetect			=> rx_ctrldetect,
		rx_dataout				=> rx_out,
		rx_disperr				=> rx_disperr,
		rx_errdetect			=> rx_errdetect,
		rx_freqlocked			=> rx_freqlocked,
		rx_valid					=> rx_valid,
		encoder_error			=> encoder_error,	
		decoder_error			=> decoder_error_i,	
		Aligned					=> Word_Aligned,
		tx_ready	 				=> tx_ready,
		rx_ready					=> rx_ready,
		reconfig_reset		      => reconfig_reset,
		reconfig_write          => reconfig_write,
		reconfig_read           => reconfig_read,
		reconfig_address        => reconfig_address,
		reconfig_writedata      => reconfig_writedata,
		reconfig_readdata       => reconfig_readdata,
		reconfig_waitrequest    => reconfig_waitrequest,
		reconfig_pll_reset       	=> reconfig_pll_reset,
		reconfig_pll_write       	=> reconfig_pll_write,
		reconfig_pll_read        	=> reconfig_pll_read,
		reconfig_pll_address        => reconfig_pll_address,
		reconfig_pll_writedata      => reconfig_pll_writedata,
		reconfig_pll_readdata       => reconfig_pll_readdata,
		reconfig_pll_waitrequest 	=> reconfig_pll_waitrequest		
	);


	
-----------------------------------------------------------------------------------------------------------
--	Generate Reset_I synchronously to DataOut_Clock
-----------------------------------------------------------------------------------------------------------	

Reset_in2 <= Reset or not (rx_ready); -- Both or generated on the CLk100Mhz clock domain.

Reset_Synchro_inst2 : Reset_Synchro
PORT MAP
	(
	Clk			=> DataOut_Clock,
	Reset_in		=> Reset_in2,
	Reset_out	=> Reset_I
	);


Reset_Rx <= Reset_I;	

-----------------------------------------------------------------------------------------------------------
--	Generate WordAligned and Reset_Rx_Path
-----------------------------------------------------------------------------------------------------------	
	

process(DataOut_Clock,Reset_Rx)
begin
	if Reset_Rx = '1' then
		WordAligned 	<= '0';
		Reset_Rx_Path 	<= '1';
		ForceAlign_min2  <= '0';
		ForceAlign_min1  <= '0';
		ForceAlign_sync	 <= '0';			
	elsif rising_edge(DataOut_Clock) then
	
		ForceAlign_min2 	<= ForceAlign;
		ForceAlign_min1	 	<= ForceAlign_min2;
		ForceAlign_sync		<= ForceAlign_min1;		-- Synchronize from Clk50 Mhz to rx_coreclk domain
		
		if Word_Aligned = Ones then
			WordAligned <= '1';
		else
			WordAligned <= '0';
		end if;
		if ((WordAligned = '1') and (ForceAlign_sync = '0')) then
			Reset_Rx_Path <= '0';
		else
			Reset_Rx_Path <= '1';
		end if;
	end if;
end process;



DataOut_Aligned <= rx_out;

Generate_Ctrl_Aligned:
FOR i IN 0 to NUMBER_OF_LANES-1 GENERATE
	Ctrl_Aligned(I) <= rx_ctrldetect(I*8);
END GENERATE;


----------------------------------------------------------------------------------------
-- Rx Path Control (Deskew Lanes)
----------------------------------------------------------------------------------------


Rx_Path_inst: Rx_Path_Deskew 
	GENERIC MAP
	(
		NUMBER_OF_LANES		=> NUMBER_OF_LANES,	  	-- NUMBER_OF_LANES
		LANEWIDTH				=> LANEWIDTH		-- LANEWIDTH for transceiver
	)
	PORT MAP( 
	Clock							=> DataOut_Clock,										
	Reset							=> Reset_Rx_Path,	
	--Enable						=> rx_valid,	
	data_in						=> DataOut_Aligned, 
	ctrl_in						=> Ctrl_Aligned,						
	data_out						=> DataOut,
	rx_fifo_rd_en				=> rx_fifo_rd_en,
	rx_enh_fifo_pempty		=> rx_enh_fifo_pempty,
	Valid_Data					=> DataOut_Valid,
	Error							=> Error_Deskew,
	Force_Rx_Reset				=> Force_Rx_Reset,	
	Lane_Aligned				=> LaneAligned,
	Linkup						=> Linkup,
	XOFF_Received				=> XOFF_Received,
	Rx_Error						=> Rx_Error,	
	Delay							=> Delay
	);

decoder_error <= '0' when (decoder_error_i = ZEROES) else '1';


			
end;



