----------------------------------------------------------------------------------------------------
-- (c)2011 Altera Corporation. All rights reserved.
--
-- Altera products are protected under numerous U.S. and foreign patents,
-- maskwork rights, copyrights and other intellectual property laws.
--
-- This reference design file, and your use thereof, is subject to and governed
-- by the terms and conditions of the applicable Altera Reference Design License
-- Agreement (either as signed by you or found at www.altera.com).  By using
-- this reference design file, you indicate your acceptance of such terms and
-- conditions between you and Altera Corporation.  In the event that you do not
-- agree with such terms and conditions, you may not use the reference design
-- file and please promptly destroy any copies you have made.
--
-- This reference design file is being provided on an "as-is�" basis and as an
-- accommodation and therefore all warranties, representations or guarantees of
-- any kind (whether express, implied or statutory) including, without
-- limitation, warranties of merchantability, non-infringement, or fitness for
-- a particular purpose, are specifically disclaimed.  By making this reference
-- design file available, Altera expressly does not recommend, suggest or
-- require that this reference design file be used in combination with any
-- other product not provided by Altera.
----------------------------------------------------------------------------------------------------
-- File          : Tx_Ratematcher_Control.vhd
-- Author		 : Peter Schepers
----------------------------------------------------------------------------------------------------
--
-- SuperLite Tx Module : 	- Contains the control logic to control the ratematcher.
--							- Instantiates the Tx_Ratematcher FIFO (32 words deep)
----------------------------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
 
entity Tx_Ratematcher_Control is
GENERIC
	(
		READ_LENGTH							: std_logic_vector(15 downto 0) := X"00FF";
		IDLE_LENGTH							: std_logic_vector(3 downto 0)  := X"7";	
		NUMBER_OF_LANES					: integer := 11;	  									-- Number of lanes
		LANEWIDTH							: integer := 64;										-- LANEWIDTH for transceiver
		RMFIFO_READ_ON_TH  				: std_logic_vector(4 downto 0) := "10100"; 	-- Ratematching fifo read on threshold: read starts when rlev is greater than this threshold
		RMFIFO_READ_OFF_TH 				: std_logic_vector(4 downto 0) := "00011" 	-- Ratematching fifo read off threshold: read stops when rlev is less than this threshold
	);
PORT(
	data_in						: in std_logic_vector(((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);
	data_in_valid				: in std_logic;
	data_in_ready				: out std_logic;	
	wrclk							: in std_logic;							
	rdclk							: in std_logic;							
	Reset							: in std_logic;			-- Asynchronous to wrclk and rdclk				
   LaneAligned					: in std_logic; 			-- LaneAligned status from the receiver (clocked by recovered clock)
   XOFF							: in std_logic; 			-- XOFF input to sent out XOFF to the remote device for the remote to stop sending traffic (100 Mhz Clock domain)	
	Stop_Traffic				: in std_logic;			-- Used to stop sending traffic locally (when XOFF is received from remote side) (clocked by recovered clock)
	Tx_Data						: out std_logic_vector(((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);
	Tx_Ctrlenable				: out std_logic_vector(((NUMBER_OF_LANES * (LANEWIDTH/8))-1) downto 0);
	wrfull_Tx_Ratematcher	: out std_logic
	);
END Tx_Ratematcher_Control;	

architecture rtl of Tx_Ratematcher_Control is


component tx_ratematcher
	PORT
	(
		aclr		: IN STD_LOGIC  := '0';
		data		: IN STD_LOGIC_VECTOR (((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);
		rdclk		: IN STD_LOGIC ;		
		rdreq		: IN STD_LOGIC ;
		wrclk		: IN STD_LOGIC ;		
		wrreq		: IN STD_LOGIC ;
		q		: OUT STD_LOGIC_VECTOR(((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);
		rdempty		: OUT STD_LOGIC ;
		rdfull		: OUT STD_LOGIC ;
		rdusedw		: OUT STD_LOGIC_VECTOR (4 DOWNTO 0);
		wrfull		: OUT STD_LOGIC ;
		wrusedw		: OUT STD_LOGIC_VECTOR (4 DOWNTO 0)
	);
end component;

component Reset_Synchro
port
	(
	Clk			: in std_logic;
	Reset_in	: in std_logic;
	Reset_out	: out std_logic
	);
END component;

component Synchro is
port
	(
	Clk			: in std_logic;
	data_in		: in std_logic;
	data_out		: out std_logic
	);
END component;	
	

signal Tx_Valid					:  std_logic;	

type   TxState	 is (Train_Data,Train_Control,Train_Align,Tx_ResetFifo,WriteOnly,ReadAndWrite,ReadOnly,Stop_Sending_Data);
signal Tx_FifoState	: TxState;



signal Ready_Tx_Ratematcher	: std_logic;
signal data_in_ready_min1   	: std_logic;
signal data_in_ready_min2   	: std_logic;

signal Reset_tx_Ratematcher		: std_logic;
signal wrusedw_Tx_Ratematcher 	: std_logic_vector(4 downto 0);
signal rdusedw_Tx_Ratematcher 	: std_logic_vector(4 downto 0);
signal rdempty_Tx_Ratematcher 	: std_logic;
signal RdEna_Tx_Ratematcher 		: std_logic;
signal wrfull 							: std_logic;
signal data_out 						: std_logic_vector(((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);
signal Reset_rdclk 					: std_logic;
signal Reset_wrclk					: std_logic;

signal IdleLength	 					: std_logic_vector(3 downto 0);
signal ReadLength						: std_logic_vector(15 downto 0);
signal DataCounter					: std_logic_vector(3 downto 0);
signal IdleCounter					: std_logic_vector(2 downto 0);

signal XOFF_rdclk						: std_logic;
signal Stop_Traffic_rdclk			: std_logic;
signal LaneAligned_rdclk			: std_logic;
signal count							: std_logic_vector(4 downto 0);


begin

-----------------------------------------------------------------------------------------------------------
--	Synchronize input signals to rdclk domain of TxFIFO
-----------------------------------------------------------------------------------------------------------	
Syncho_inst1 : Synchro
PORT MAP
(
	Clk			=> rdclk,
	data_in		=> XOFF,
	data_out		=> XOFF_rdclk
);

Syncho_inst2 : Synchro
PORT MAP
(
	Clk			=> rdclk,
	data_in		=> Stop_Traffic,
	data_out		=> Stop_Traffic_rdclk
);

Syncho_inst3 : Synchro
PORT MAP
(
	Clk			=> rdclk,
	data_in		=> LaneAligned,
	data_out		=> LaneAligned_rdclk
);

-----------------------------------------------------------------------------------------------------------
--	Synchronize Reset to rdclk
-----------------------------------------------------------------------------------------------------------	

Reset_Synchro_inst : Reset_Synchro
PORT MAP
	(
	Clk			=> rdclk,
	Reset_in		=> Reset,
	Reset_out	=> Reset_rdclk
	);
	
-----------------------------------------------------------------------------------------------------------
--	Instantiate Tx_Ratematcher
-----------------------------------------------------------------------------------------------------------	

Tx_Ratematcher_inst : Tx_Ratematcher PORT MAP (
		data	 => data_in,
		wrreq	 => data_in_valid,
		rdreq	 => RdEna_Tx_Ratematcher,
		rdclk	 => rdclk,
		wrclk	 => wrclk,
		aclr	 => Reset_rdclk, -- Synchronous with rdclk, FIFO is instantiated with aclr additionally registered to wrclk domain
		q	 	 => data_out,
		rdempty	 => rdempty_Tx_Ratematcher ,
		rdusedw	=>  rdusedw_Tx_Ratematcher,
		wrfull	 => wrfull,
		wrusedw	 => wrusedw_Tx_Ratematcher --not used
	);

	


-----------------------------------------------------------------------------------------------------------
--	Tx_Ratematcher Fifo Control
-----------------------------------------------------------------------------------------------------------	

	process (rdclk,Reset_rdclk)
	begin
		if Reset_rdclk = '1' then
				Tx_FifoState <= Tx_ResetFifo;	
				ReadLength	 <= (OTHERS => '0');
				IdleLength	 <= (OTHERS => '0');	
				DataCounter  <= (OTHERS => '0');	
				IdleCounter	 <= (OTHERS => '0');
		elsif rdclk'event and rdclk = '1' then

		     CASE Tx_FifoState IS
		         WHEN Tx_ResetFifo    		=> 	Tx_FifoState 	<= Train_Data;
																ReadLength		<= (OTHERS => '0');
																IdleLength		<= (OTHERS => '0');
																
					WHEN Train_Data		 		=>  if rdusedw_Tx_Ratematcher >= RMFIFO_READ_ON_TH then  -- Almost Full
															Tx_FifoState <= ReadAndWrite;
														 end if;
														 if DataCounter = "1111" then
															Tx_FifoState <= Train_Control;
															DataCounter  <= (OTHERS => '0');	
														 else
															DataCounter <= DataCounter + 1;
														 end if;
														 ReadLength <= (OTHERS => '0');
														 IdleLength	<= (OTHERS => '0');

					WHEN Train_Control		 	 =>  if rdusedw_Tx_Ratematcher >= RMFIFO_READ_ON_TH then  -- Almost Full
															Tx_FifoState <= ReadAndWrite;
														 end if;
														 if Idlecounter = "111" then
															Tx_FifoState <= Train_Align;
															Idlecounter  <= (OTHERS => '0');	
														 else
															Idlecounter <= Idlecounter + 1;
														 end if;
														 ReadLength <= (OTHERS => '0');
														 IdleLength	<= (OTHERS => '0');														 

					WHEN Train_Align		 	 =>  Tx_FifoState <= Train_Data;
														 ReadLength <= (OTHERS => '0');
														 IdleLength	<= (OTHERS => '0');
														 
					WHEN WriteOnly		 		=>  if Stop_Traffic_rdclk = '0' then
															if rdusedw_Tx_Ratematcher >= RMFIFO_READ_ON_TH then  -- Almost Full
																Tx_FifoState <= ReadAndWrite;
															end if;
														 else
															 Tx_FifoState <= Stop_Sending_Data;
														 end if;
														 ReadLength <= (OTHERS => '0');
														 IdleLength	<= (OTHERS => '0');
															
				   WHEN ReadAndWrite			=>  if Stop_Traffic_rdclk = '0' then	
																if (rdusedw_Tx_Ratematcher <= RMFIFO_READ_OFF_TH) then  -- Almost Empty
																	Tx_FifoState <= WriteOnly;
																elsif (ReadLength >= READ_LENGTH) then  
																	Tx_FifoState <= ReadOnly;	
																	ReadLength 	 <= (OTHERS => '0');
																else
																	Tx_FifoState <= ReadAndWrite;
																	ReadLength	 <= ReadLength + 1;
																end if;	
														  else
															 Tx_FifoState <= Stop_Sending_Data;
														  end if;																
															IdleLength <= (OTHERS => '0');
					WHEN ReadOnly				=>  if Stop_Traffic_rdclk = '0' then  
																if IdleLength = IDLE_LENGTH then
																	Tx_FifoState <= ReadAndWrite;
																	IdleLength	 <= (OTHERS => '0');
																else
																	Tx_FifoState	<= ReadOnly;
																	IdleLength		<= IdleLength + 1;
																end if;
														  else
															 Tx_FifoState <= Stop_Sending_Data;
														  end if;
					WHEN Stop_Sending_Data				=>  if Stop_Traffic_rdclk = '0' then  
																if (rdusedw_Tx_Ratematcher <= RMFIFO_READ_OFF_TH) then  -- Almost Empty
																	Tx_FifoState <= WriteOnly;
																elsif (ReadLength >= READ_LENGTH) then  
																	Tx_FifoState <= ReadOnly;	
																	ReadLength 	 <= (OTHERS => '0');
																else
																	Tx_FifoState <= ReadAndWrite;
																	ReadLength	 <= ReadLength + 1;
																end if;	
														  else
															 Tx_FifoState <= Stop_Sending_Data;
														  end if;
														  
					When OTHERS					=> 	Tx_FifoState <= Tx_ResetFifo;
																
				end case;								
		end if;
   	end process;


	process (rdclk,Reset_rdclk)
	begin
		if Reset_rdclk = '1' then
				RdEna_Tx_Ratematcher 	<= '0';
				Ready_Tx_Ratematcher 	<= '0';
				Tx_Valid			 	 		<= '0';
				Tx_CtrlEnable		 		<= (OTHERS => '1');
				FOR i IN 0 to (NUMBER_OF_LANES-1) LOOP
					Tx_Data((LANEWIDTH*(I+1)-1) downto (LANEWIDTH*I)) <= X"1C1C1C1C1C1C1CBC"; -- K28.5 contained in lowest byte
				END LOOP;	
		elsif rdclk'event and rdclk = '1' then
		       CASE Tx_FifoState IS
		         WHEN Tx_ResetFifo    	=> 	
												Ready_Tx_Ratematcher <= '0';
												RdEna_Tx_Ratematcher <= '0';

					WHEN Train_Data	 	=>	
												Ready_Tx_Ratematcher <= '1';	
												RdEna_Tx_Ratematcher <= '0';	

					WHEN Train_Control	 	=>	
												Ready_Tx_Ratematcher <= '1';	
												RdEna_Tx_Ratematcher <= '0';	

					WHEN Train_Align	 	=>	
												Ready_Tx_Ratematcher <= '1';	
												RdEna_Tx_Ratematcher <= '0';													
												
					WHEN WriteOnly	 	=>	
												Ready_Tx_Ratematcher <= '1';	
												RdEna_Tx_Ratematcher <= '0';										
																				
					WHEN ReadAndWrite		=>	
												Ready_Tx_Ratematcher <= '1';									
												RdEna_Tx_Ratematcher <= '1';
												
					WHEN ReadOnly		=>	
												Ready_Tx_Ratematcher <= '0';									
												RdEna_Tx_Ratematcher <= '1';	
	
					WHEN Stop_Sending_Data		=>	
												Ready_Tx_Ratematcher <= '0';									
												RdEna_Tx_Ratematcher <= '0';	
																								
					When OTHERS				=> 	
												Ready_Tx_Ratematcher <= '0';
												RdEna_Tx_Ratematcher <= '0';
				end case;		

				--Tx_Valid <= RdEna_Tx_Ratematcher; -- RdEna_Tx_Ratematcher delayed for one clock cycle
				
				if (RdEna_Tx_Ratematcher = '1' and rdempty_Tx_Ratematcher = '0' ) then -- Protect against rdempty
					Tx_Valid <= '1';
				else
					Tx_Valid <= '0';
				end if;
				
				if Tx_Valid = '0' then
					if Tx_FifoState = Train_Data then 
						FOR i IN 0 to (NUMBER_OF_LANES-2) LOOP
							Tx_Data((LANEWIDTH*(I+1)-1) downto (LANEWIDTH*I)) <= X"AAAAAAAAAAAAAAAA"; -- Training Data
						END LOOP;
						Tx_Data((LANEWIDTH*(NUMBER_OF_LANES)-1) downto (LANEWIDTH*(NUMBER_OF_LANES-1))) <= (OTHERS => '0');
						Tx_CtrlEnable <= (OTHERS => '0');
					elsif Tx_FifoState = Train_Control then 
						FOR i IN 0 to (NUMBER_OF_LANES-1) LOOP
							Tx_Data((LANEWIDTH*(I+1)-1) downto (LANEWIDTH*I)) <= X"1C1C1C1C1C1C1CBC"; -- K28.5 contained in lowest byte
						END LOOP;
						Tx_CtrlEnable <= (OTHERS => '1');
					elsif Tx_FifoState = Train_Align then
						FOR i IN 0 to (NUMBER_OF_LANES-1) LOOP
							Tx_Data((LANEWIDTH*(I+1)-1) downto (LANEWIDTH*I)) <= X"7C7C7C7C7C7C7C7C"; -- Alignment character
						END LOOP;				
						Tx_CtrlEnable <= (OTHERS => '1');				
				   elsif Tx_FifoState = Stop_Sending_Data then
						 if count(4) = '1' then
							count <= (Others => '0');
								if (XOFF_rdclk = '1') then
									FOR i IN 0 to (NUMBER_OF_LANES-1) LOOP
										Tx_Data((LANEWIDTH*(I+1)-1) downto (LANEWIDTH*I)) <= X"7C7C7C7C7C7C9C7C"; -- sent out once in a while that traffic has been stopped sending and XOFF is requested
									END LOOP;
								else
									FOR i IN 0 to (NUMBER_OF_LANES-1) LOOP
										Tx_Data((LANEWIDTH*(I+1)-1) downto (LANEWIDTH*I)) <= X"7C7C7C7C7C7C3C7C"; -- sent out once in a while that traffic has been stopped sending.
									END LOOP;
								end if;
						 else
							count <= count + 1;
							FOR i IN 0 to (NUMBER_OF_LANES-1) LOOP
								Tx_Data((LANEWIDTH*(I+1)-1) downto (LANEWIDTH*I)) <= X"1C1C1C1C1C1C1CBC"; -- K28.5 contained in lowest byte
							END LOOP;							
						 end if;
						 Tx_CtrlEnable <= (OTHERS => '1');
					elsif (RdEna_Tx_Ratematcher = '1'  and rdempty_Tx_Ratematcher = '0') then -- Last control word should be alignment character
						if (LaneAligned_rdclk = '1' and XOFF_rdclk = '0') then
								FOR i IN 0 to (NUMBER_OF_LANES-1) LOOP
									Tx_Data((LANEWIDTH*(I+1)-1) downto (LANEWIDTH*I)) <= X"7C7C7C7C7C7CFD7C"; -- Alignment character with LaneAligned achieved no XOFF -- K29.7/K28.3
								END LOOP;
								Tx_CtrlEnable <= (OTHERS => '1');
						elsif (LaneAligned_rdclk = '1' and XOFF_rdclk = '1') then
								FOR i IN 0 to (NUMBER_OF_LANES-1) LOOP
									Tx_Data((LANEWIDTH*(I+1)-1) downto (LANEWIDTH*I)) <= X"7C7C7C7C7C7C5C7C"; -- Alignment character with LaneAligned achieved and XOFF active - K28.2/K28.3
								END LOOP;
								Tx_CtrlEnable <= (OTHERS => '1');
						else
								FOR i IN 0 to (NUMBER_OF_LANES-1) LOOP
									Tx_Data((LANEWIDTH*(I+1)-1) downto (LANEWIDTH*I)) <= X"7C7C7C7C7C7C7C7C"; -- Alignment character
								END LOOP;
								Tx_CtrlEnable <= (OTHERS => '1');
						end if;	
					else
						FOR i IN 0 to (NUMBER_OF_LANES-1) LOOP
							Tx_Data((LANEWIDTH*(I+1)-1) downto (LANEWIDTH*I)) <= X"1C1C1C1C1C1C1CBC"; -- K28.0/K28.5 
						END LOOP;
						Tx_CtrlEnable <= (OTHERS => '1');
					end if;
				else

						Tx_Data <= data_out;
						Tx_CtrlEnable <= (OTHERS => '0');
				end if;
				


				
		end if;
   	end process;
				

-----------------------------------------------------------------------------------------------------------
--	Synchronize Reset to wrclk and generate data_in_ready
-----------------------------------------------------------------------------------------------------------	

Reset_Synchro_inst2 : Reset_Synchro
PORT MAP
	(
	Clk			=> wrclk,
	Reset_in		=> Reset,
	Reset_out	=> Reset_wrclk
	);

	process (wrclk,Reset_wrclk)
	begin
		if Reset_wrclk = '1' then
				data_in_ready				<= '0';
				data_in_ready_min1 		<= '0';
				data_in_ready_min2 		<= '0';
		elsif wrclk'event and wrclk = '1' then
				data_in_ready_min2 		<= Ready_Tx_Ratematcher ; -- synchronize to Write clock domain
				data_in_ready_min1 		<= data_in_ready_min2;
				data_in_ready 				<= data_in_ready_min1;
		end if;
   	end process;

wrfull_Tx_Ratematcher <= wrfull;
					
end;



